@@ -24,8 +24,9 @@ import (
 
 	"github.com/golang/mock/gomock"
 	"github.com/stretchr/testify/assert"
-	"k8s.io/kubernetes/pkg/api/v1"
-	runtimeapi "k8s.io/kubernetes/pkg/kubelet/apis/cri/v1alpha1/runtime"
+	v1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/types"
+	runtimeapi "k8s.io/cri-api/pkg/apis/runtime/v1"
 	kubecontainer "k8s.io/kubernetes/pkg/kubelet/container"
 	containertest "k8s.io/kubernetes/pkg/kubelet/container/testing"
 )
@@ -34,17 +35,15 @@ func TestSandboxGC(t *testing.T) {
 	fakeRuntime, _, m, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 
-	fakePodGetter := m.containerGC.podGetter.(*fakePodGetter)
-	makeGCSandbox := func(pod *v1.Pod, attempt uint32, state runtimeapi.PodSandboxState, withPodGetter bool, createdAt int64) sandboxTemplate {
-		if withPodGetter {
-			// initialize the pod getter
-			fakePodGetter.pods[pod.UID] = pod
-		}
+	podStateProvider := m.containerGC.podStateProvider.(*fakePodStateProvider)
+	makeGCSandbox := func(pod *v1.Pod, attempt uint32, state runtimeapi.PodSandboxState, hasRunningContainers, isTerminating bool, createdAt int64) sandboxTemplate {
 		return sandboxTemplate{
-			pod:       pod,
-			state:     state,
-			attempt:   attempt,
-			createdAt: createdAt,
+			pod:         pod,
+			state:       state,
+			attempt:     attempt,
+			createdAt:   createdAt,
+			running:     hasRunningContainers,
+			terminating: isTerminating,
 		}
 	}
 
@@ -61,100 +60,144 @@ func TestSandboxGC(t *testing.T) {
 		}),
 	}
 
-	for c, test := range []struct {
-		description         string              // description of the test case
-		sandboxes           []sandboxTemplate   // templates of sandboxes
-		containers          []containerTemplate // templates of containers
-		minAge              time.Duration       // sandboxMinGCAge
-		remain              []int               // template indexes of remaining sandboxes
-		evictTerminatedPods bool
+	for _, test := range []struct {
+		description          string              // description of the test case
+		sandboxes            []sandboxTemplate   // templates of sandboxes
+		containers           []containerTemplate // templates of containers
+		remain               []int               // template indexes of remaining sandboxes
+		evictTerminatingPods bool
 	}{
 		{
 			description: "notready sandboxes without containers for deleted pods should be garbage collected.",
 			sandboxes: []sandboxTemplate{
-				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, 0),
+				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, false, 0),
 			},
-			containers:          []containerTemplate{},
-			remain:              []int{},
-			evictTerminatedPods: false,
+			containers:           []containerTemplate{},
+			remain:               []int{},
+			evictTerminatingPods: false,
 		},
 		{
 			description: "ready sandboxes without containers for deleted pods should not be garbage collected.",
 			sandboxes: []sandboxTemplate{
-				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_READY, false, 0),
+				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_READY, false, false, 0),
 			},
-			containers:          []containerTemplate{},
-			remain:              []int{0},
-			evictTerminatedPods: false,
+			containers:           []containerTemplate{},
+			remain:               []int{0},
+			evictTerminatingPods: false,
 		},
 		{
 			description: "sandboxes for existing pods should not be garbage collected.",
 			sandboxes: []sandboxTemplate{
-				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_READY, true, 0),
-				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, 0),
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_READY, true, false, 0),
+				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 0),
 			},
-			containers:          []containerTemplate{},
-			remain:              []int{0, 1},
-			evictTerminatedPods: false,
+			containers:           []containerTemplate{},
+			remain:               []int{0, 1},
+			evictTerminatingPods: false,
 		},
 		{
-			description: "non-running sandboxes for existing pods should be garbage collected if evictTerminatedPods is set.",
+			description: "older exited sandboxes without containers for existing pods should be garbage collected if there are more than one exited sandboxes.",
 			sandboxes: []sandboxTemplate{
-				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_READY, true, 0),
-				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, 0),
+				makeGCSandbox(pods[0], 1, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 1),
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 0),
 			},
-			containers:          []containerTemplate{},
-			remain:              []int{0},
-			evictTerminatedPods: true,
+			containers:           []containerTemplate{},
+			remain:               []int{0},
+			evictTerminatingPods: false,
+		},
+		{
+			description: "older exited sandboxes with containers for existing pods should not be garbage collected even if there are more than one exited sandboxes.",
+			sandboxes: []sandboxTemplate{
+				makeGCSandbox(pods[0], 1, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 1),
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 0),
+			},
+			containers: []containerTemplate{
+				{pod: pods[0], container: &pods[0].Spec.Containers[0], sandboxAttempt: 0, state: runtimeapi.ContainerState_CONTAINER_EXITED},
+			},
+			remain:               []int{0, 1},
+			evictTerminatingPods: false,
+		},
+		{
+			description: "non-running sandboxes for existing pods should be garbage collected if evictTerminatingPods is set.",
+			sandboxes: []sandboxTemplate{
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_READY, true, true, 0),
+				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, true, 0),
+			},
+			containers:           []containerTemplate{},
+			remain:               []int{0},
+			evictTerminatingPods: true,
 		},
 		{
 			description: "sandbox with containers should not be garbage collected.",
 			sandboxes: []sandboxTemplate{
-				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, 0),
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, false, 0),
 			},
 			containers: []containerTemplate{
 				{pod: pods[0], container: &pods[0].Spec.Containers[0], state: runtimeapi.ContainerState_CONTAINER_EXITED},
 			},
-			remain:              []int{0},
-			evictTerminatedPods: false,
+			remain:               []int{0},
+			evictTerminatingPods: false,
 		},
 		{
 			description: "multiple sandboxes should be handled properly.",
 			sandboxes: []sandboxTemplate{
 				// running sandbox.
-				makeGCSandbox(pods[0], 1, runtimeapi.PodSandboxState_SANDBOX_READY, true, 1),
+				makeGCSandbox(pods[0], 1, runtimeapi.PodSandboxState_SANDBOX_READY, true, false, 1),
 				// exited sandbox without containers.
-				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, 0),
+				makeGCSandbox(pods[0], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 0),
 				// exited sandbox with containers.
-				makeGCSandbox(pods[1], 1, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, 1),
+				makeGCSandbox(pods[1], 1, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 1),
 				// exited sandbox without containers.
-				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, 0),
+				makeGCSandbox(pods[1], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, true, false, 0),
 				// exited sandbox without containers for deleted pods.
-				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, 0),
+				makeGCSandbox(pods[2], 0, runtimeapi.PodSandboxState_SANDBOX_NOTREADY, false, true, 0),
 			},
 			containers: []containerTemplate{
 				{pod: pods[1], container: &pods[1].Spec.Containers[0], sandboxAttempt: 1, state: runtimeapi.ContainerState_CONTAINER_EXITED},
 			},
-			remain:              []int{0, 2},
-			evictTerminatedPods: false,
+			remain:               []int{0, 2},
+			evictTerminatingPods: false,
 		},
 	} {
-		t.Logf("TestCase #%d: %+v", c, test)
-		fakeSandboxes := makeFakePodSandboxes(t, m, test.sandboxes)
-		fakeContainers := makeFakeContainers(t, m, test.containers)
-		fakeRuntime.SetFakeSandboxes(fakeSandboxes)
-		fakeRuntime.SetFakeContainers(fakeContainers)
+		t.Run(test.description, func(t *testing.T) {
+			podStateProvider.removed = make(map[types.UID]struct{})
+			podStateProvider.terminated = make(map[types.UID]struct{})
+			fakeSandboxes := makeFakePodSandboxes(t, m, test.sandboxes)
+			fakeContainers := makeFakeContainers(t, m, test.containers)
+			for _, s := range test.sandboxes {
+				if !s.running && s.pod.Name == "deleted" {
+					podStateProvider.removed[s.pod.UID] = struct{}{}
+				}
+				if s.terminating {
+					podStateProvider.terminated[s.pod.UID] = struct{}{}
+				}
+			}
+			fakeRuntime.SetFakeSandboxes(fakeSandboxes)
+			fakeRuntime.SetFakeContainers(fakeContainers)
 
-		err := m.containerGC.evictSandboxes(test.evictTerminatedPods)
-		assert.NoError(t, err)
-		realRemain, err := fakeRuntime.ListPodSandbox(nil)
-		assert.NoError(t, err)
-		assert.Len(t, realRemain, len(test.remain))
-		for _, remain := range test.remain {
-			status, err := fakeRuntime.PodSandboxStatus(fakeSandboxes[remain].Id)
+			err := m.containerGC.evictSandboxes(test.evictTerminatingPods)
 			assert.NoError(t, err)
-			assert.Equal(t, &fakeSandboxes[remain].PodSandboxStatus, status)
-		}
+			realRemain, err := fakeRuntime.ListPodSandbox(nil)
+			assert.NoError(t, err)
+			assert.Len(t, realRemain, len(test.remain))
+			for _, remain := range test.remain {
+				status, err := fakeRuntime.PodSandboxStatus(fakeSandboxes[remain].Id)
+				assert.NoError(t, err)
+				assert.Equal(t, &fakeSandboxes[remain].PodSandboxStatus, status)
+			}
+		})
+	}
+}
+
+func makeGCContainer(podName, containerName string, attempt int, createdAt int64, state runtimeapi.ContainerState) containerTemplate {
+	container := makeTestContainer(containerName, "test-image")
+	pod := makeTestPod(podName, "test-ns", podName, []v1.Container{container})
+	return containerTemplate{
+		pod:       pod,
+		container: &container,
+		attempt:   attempt,
+		createdAt: createdAt,
+		state:     state,
 	}
 }
 
@@ -162,43 +205,26 @@ func TestContainerGC(t *testing.T) {
 	fakeRuntime, _, m, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 
-	fakePodGetter := m.containerGC.podGetter.(*fakePodGetter)
 	podStateProvider := m.containerGC.podStateProvider.(*fakePodStateProvider)
-	makeGCContainer := func(podName, containerName string, attempt int, createdAt int64, state runtimeapi.ContainerState) containerTemplate {
-		container := makeTestContainer(containerName, "test-image")
-		pod := makeTestPod(podName, "test-ns", podName, []v1.Container{container})
-		if podName == "running" {
-			podStateProvider.runningPods[pod.UID] = struct{}{}
-		}
-		if podName != "deleted" {
-			// initialize the pod getter, explicitly exclude deleted pod
-			fakePodGetter.pods[pod.UID] = pod
-		}
-		return containerTemplate{
-			pod:       pod,
-			container: &container,
-			attempt:   attempt,
-			createdAt: createdAt,
-			state:     state,
-		}
-	}
-	defaultGCPolicy := kubecontainer.ContainerGCPolicy{MinAge: time.Hour, MaxPerPodContainer: 2, MaxContainers: 6}
+	defaultGCPolicy := kubecontainer.GCPolicy{MinAge: time.Hour, MaxPerPodContainer: 2, MaxContainers: 6}
 
-	for c, test := range []struct {
-		description         string                           // description of the test case
-		containers          []containerTemplate              // templates of containers
-		policy              *kubecontainer.ContainerGCPolicy // container gc policy
-		remain              []int                            // template indexes of remaining containers
-		evictTerminatedPods bool
+	for _, test := range []struct {
+		description          string                  // description of the test case
+		containers           []containerTemplate     // templates of containers
+		policy               *kubecontainer.GCPolicy // container gc policy
+		remain               []int                   // template indexes of remaining containers
+		evictTerminatingPods bool
+		allSourcesReady      bool
 	}{
 		{
 			description: "all containers should be removed when max container limit is 0",
 			containers: []containerTemplate{
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			policy:              &kubecontainer.ContainerGCPolicy{MinAge: time.Minute, MaxPerPodContainer: 1, MaxContainers: 0},
-			remain:              []int{},
-			evictTerminatedPods: false,
+			policy:               &kubecontainer.GCPolicy{MinAge: time.Minute, MaxPerPodContainer: 1, MaxContainers: 0},
+			remain:               []int{},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "max containers should be complied when no max per pod container limit is set",
@@ -209,9 +235,10 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			policy:              &kubecontainer.ContainerGCPolicy{MinAge: time.Minute, MaxPerPodContainer: -1, MaxContainers: 4},
-			remain:              []int{0, 1, 2, 3},
-			evictTerminatedPods: false,
+			policy:               &kubecontainer.GCPolicy{MinAge: time.Minute, MaxPerPodContainer: -1, MaxContainers: 4},
+			remain:               []int{0, 1, 2, 3},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "no containers should be removed if both max container and per pod container limits are not set",
@@ -220,9 +247,10 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			policy:              &kubecontainer.ContainerGCPolicy{MinAge: time.Minute, MaxPerPodContainer: -1, MaxContainers: -1},
-			remain:              []int{0, 1, 2},
-			evictTerminatedPods: false,
+			policy:               &kubecontainer.GCPolicy{MinAge: time.Minute, MaxPerPodContainer: -1, MaxContainers: -1},
+			remain:               []int{0, 1, 2},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "recently started containers should not be removed",
@@ -231,8 +259,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, time.Now().UnixNano(), runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, time.Now().UnixNano(), runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 1, 2},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1, 2},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "oldest containers should be removed when per pod container limit exceeded",
@@ -241,8 +270,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 1},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "running containers should not be removed",
@@ -251,8 +281,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_RUNNING),
 			},
-			remain:              []int{0, 1, 2},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1, 2},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "no containers should be removed when limits are not exceeded",
@@ -260,8 +291,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 1},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "max container count should apply per (UID, container) pair",
@@ -276,8 +308,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo2", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo2", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 1, 3, 4, 6, 7},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1, 3, 4, 6, 7},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "max limit should apply and try to keep from every pod",
@@ -293,8 +326,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo4", "bar4", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo4", "bar4", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 2, 4, 6, 8},
-			evictTerminatedPods: false,
+			remain:               []int{0, 2, 4, 6, 8},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
 			description: "oldest pods should be removed if limit exceeded",
@@ -310,11 +344,12 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("foo6", "bar6", 2, 2, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo7", "bar7", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 2, 4, 6, 8, 9},
-			evictTerminatedPods: false,
+			remain:               []int{0, 2, 4, 6, 8, 9},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
 		},
 		{
-			description: "all non-running containers should be removed when evictTerminatedPods is set",
+			description: "all non-running containers should be removed when evictTerminatingPods is set",
 			containers: []containerTemplate{
 				makeGCContainer("foo", "bar", 2, 2, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo", "bar", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
@@ -323,8 +358,9 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("running", "bar2", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("foo3", "bar3", 0, 0, runtimeapi.ContainerState_CONTAINER_RUNNING),
 			},
-			remain:              []int{4, 5},
-			evictTerminatedPods: true,
+			remain:               []int{4, 5},
+			evictTerminatingPods: true,
+			allSourcesReady:      true,
 		},
 		{
 			description: "containers for deleted pods should be removed",
@@ -336,27 +372,48 @@ func TestContainerGC(t *testing.T) {
 				makeGCContainer("deleted", "bar1", 1, 1, runtimeapi.ContainerState_CONTAINER_EXITED),
 				makeGCContainer("deleted", "bar1", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
 			},
-			remain:              []int{0, 1, 2},
-			evictTerminatedPods: false,
+			remain:               []int{0, 1, 2},
+			evictTerminatingPods: false,
+			allSourcesReady:      true,
+		},
+		{
+			description: "containers for deleted pods may not be removed if allSourcesReady is set false ",
+			containers: []containerTemplate{
+				makeGCContainer("deleted", "bar1", 0, 0, runtimeapi.ContainerState_CONTAINER_EXITED),
+			},
+			remain:               []int{0},
+			evictTerminatingPods: true,
+			allSourcesReady:      false,
 		},
 	} {
-		t.Logf("TestCase #%d: %+v", c, test)
-		fakeContainers := makeFakeContainers(t, m, test.containers)
-		fakeRuntime.SetFakeContainers(fakeContainers)
+		t.Run(test.description, func(t *testing.T) {
+			podStateProvider.removed = make(map[types.UID]struct{})
+			podStateProvider.terminated = make(map[types.UID]struct{})
+			fakeContainers := makeFakeContainers(t, m, test.containers)
+			for _, s := range test.containers {
+				if s.pod.Name == "deleted" {
+					podStateProvider.removed[s.pod.UID] = struct{}{}
+				}
+				if s.pod.Name != "running" {
+					podStateProvider.terminated[s.pod.UID] = struct{}{}
+				}
+			}
+			fakeRuntime.SetFakeContainers(fakeContainers)
 
-		if test.policy == nil {
-			test.policy = &defaultGCPolicy
-		}
-		err := m.containerGC.evictContainers(*test.policy, true, test.evictTerminatedPods)
-		assert.NoError(t, err)
-		realRemain, err := fakeRuntime.ListContainers(nil)
-		assert.NoError(t, err)
-		assert.Len(t, realRemain, len(test.remain))
-		for _, remain := range test.remain {
-			status, err := fakeRuntime.ContainerStatus(fakeContainers[remain].Id)
+			if test.policy == nil {
+				test.policy = &defaultGCPolicy
+			}
+			err := m.containerGC.evictContainers(*test.policy, test.allSourcesReady, test.evictTerminatingPods)
 			assert.NoError(t, err)
-			assert.Equal(t, &fakeContainers[remain].ContainerStatus, status)
-		}
+			realRemain, err := fakeRuntime.ListContainers(nil)
+			assert.NoError(t, err)
+			assert.Len(t, realRemain, len(test.remain))
+			for _, remain := range test.remain {
+				status, err := fakeRuntime.ContainerStatus(fakeContainers[remain].Id)
+				assert.NoError(t, err)
+				assert.Equal(t, &fakeContainers[remain].ContainerStatus, status)
+			}
+		})
 	}
 }
 
@@ -365,16 +422,18 @@ func TestPodLogDirectoryGC(t *testing.T) {
 	_, _, m, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 	fakeOS := m.osInterface.(*containertest.FakeOS)
-	fakePodGetter := m.containerGC.podGetter.(*fakePodGetter)
 	podStateProvider := m.containerGC.podStateProvider.(*fakePodStateProvider)
 
 	// pod log directories without corresponding pods should be removed.
-	fakePodGetter.pods["123"] = makeTestPod("foo1", "new", "123", nil)
-	fakePodGetter.pods["456"] = makeTestPod("foo2", "new", "456", nil)
-	podStateProvider.runningPods["123"] = struct{}{}
-	podStateProvider.runningPods["456"] = struct{}{}
-	files := []string{"123", "456", "789", "012"}
-	removed := []string{filepath.Join(podLogsRootDirectory, "789"), filepath.Join(podLogsRootDirectory, "012")}
+	files := []string{"123", "456", "789", "012", "name_namespace_321", "name_namespace_654"}
+	removed := []string{
+		filepath.Join(podLogsRootDirectory, "789"),
+		filepath.Join(podLogsRootDirectory, "012"),
+		filepath.Join(podLogsRootDirectory, "name_namespace_654"),
+	}
+	podStateProvider.removed["012"] = struct{}{}
+	podStateProvider.removed["789"] = struct{}{}
+	podStateProvider.removed["654"] = struct{}{}
 
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
@@ -400,3 +459,26 @@ func TestPodLogDirectoryGC(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Empty(t, fakeOS.Removes)
 }
+
+func TestUnknownStateContainerGC(t *testing.T) {
+	fakeRuntime, _, m, err := createTestRuntimeManager()
+	assert.NoError(t, err)
+
+	// podStateProvider := m.containerGC.podStateProvider.(*fakePodStateProvider)
+	defaultGCPolicy := kubecontainer.GCPolicy{MinAge: time.Hour, MaxPerPodContainer: 0, MaxContainers: 0}
+
+	fakeContainers := makeFakeContainers(t, m, []containerTemplate{
+		makeGCContainer("foo", "bar", 0, 0, runtimeapi.ContainerState_CONTAINER_UNKNOWN),
+	})
+	fakeRuntime.SetFakeContainers(fakeContainers)
+
+	err = m.containerGC.evictContainers(defaultGCPolicy, true, false)
+	assert.NoError(t, err)
+
+	assert.Contains(t, fakeRuntime.GetCalls(), "StopContainer", "RemoveContainer",
+		"container in unknown state should be stopped before being removed")
+
+	remain, err := fakeRuntime.ListContainers(nil)
+	assert.NoError(t, err)
+	assert.Empty(t, remain)
+}
